
/* definitions for sonlib.h   */
/* %W% last mod on %G% at %U% */
/* Full SCCS path name: (%P%) */


#ifndef __sx11_h
#define __sx11_h

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>

#include "son.h"
#include "form.h"

/***************/
/* definitions */
/***************/

/* Parameters to SetTraceEvent */
#define MIN_EVENTS		1
#define MOTION_EVENTS		2
#define TRACE_EVENTS		3

/* Prompt definitions */
#define MENU_NONE		0
#define MAX_PROMPTS		6
#define NO_PATH			-1
#define MAX_MENU_LEVELS		2

/* Cursor definition */
#define CURSOR_STANDARD		0
#define CURSOR_MENU		1

/* Key defintions */
#define	NUMERIC_KEY	1
#define	SINGLE_KEY	2
#define	MENU_KEY	3

/* Snap definitions */
#define SNAP_NEAREST_POINT	1	/* snap to nearest point */
#define SNAP_NEAREST_SUBSECTION	2	/* snap only when cursor goes to */
					/* another subsection. */

/* Default color definition for disable menu option */
#define DEFAULT_DISABLE_TEXT_COLOR	"gray"

/* Prompt_func() defintions for typ */
#define PROMPT_CONT			1
#define PROMPT_SKIP			2
#define PROMPT_NOSKIP			3

/****************/
/* external     */
/****************/

/*********************************************/
/* structures and global/semiglobal variable */
/*********************************************/

typedef struct { 
	int id;
	KeySym keysym;
	int control;
	int meta;
	int alt;
} Key_menu_def;


struct menu_item_struct
{ 
	int x,y;           /*menu item window position*/
	unsigned int w,h;  /*menu item window size*/
	Window win;        /*window id*/
	int tx,ty;         /*text location*/
	char *str;         /*pointer to text*/
	int len;           /*length of text*/
	GC gc;             /*graphics context for text*/
	int select;        /*is the item selected?*/
	int nsub_items;    /*Number of items in submenu*/
	int enable;	   /* if TRUE menu item is enable, else user may  */
			   /* not select the item			  */
	struct menu_item_struct *sub_menu;   /*Pop up sub-menu if item selected*/
};
typedef struct menu_item_struct menu_item;

/*Data array for defining menu tree*/
typedef struct { 
	char *str;    /*menu item text*/
	int num;      /*number of items in sub-menu*/
	int button;
	int func;
	void (*subroutine)();
	int keyindex;
} menu_def;



/*************************************************
	GLOBAL PROTOTYPES: initrdb.c
*************************************************/

void 
GetInitialResourceDatabase(
#if NeedFunctionPrototypes
	Display *dpy,
	char *classname
#endif
);

XrmDatabase *
parseOpenDisp(
#if NeedFunctionPrototypes
	int *argcp,
	char *argv[],
	XrmOptionDescRec opTable[],
	int opTableEntries,
	char *name,
	char *classname
#endif
);

/*************************************************
	GLOBAL PROTOTYPES: allevent.c
*************************************************/

void
NextEvent(
#if NeedFunctionPrototypes
	Display *display,
	XEvent *report
#endif
);

void
SetTraceEventMethod(
#if NeedFunctionPrototypes
	char *param
#endif
);

void
SetTraceEvent(
#if NeedFunctionPrototypes
	int param,
	int value
#endif
);

Time 
GetLastEventTime(
#if NeedFunctionPrototypes
	void
#endif
);


/*************************************************
	GLOBAL PROTOTYPES: prompt.c
*************************************************/

void
init_prompt_array(
#if NeedFunctionPrototypes
	int debug_init
#endif
);

void
resize_prompt_array(
#if NeedFunctionPrototypes
	void
#endif
);

void
resize_panel(
#if NeedFunctionPrototypes
	menu_item *panel,  
	int num_items
#endif
);

void
map_panel(
#if NeedFunctionPrototypes
	void
#endif
);

void
unmap_panel(
#if NeedFunctionPrototypes
	void
#endif
);

void
clear_panel(
#if NeedFunctionPrototypes
	void
#endif
);

void
set_panel_cursor(
#if NeedFunctionPrototypes
	Cursor cursor
#endif
);

void
resize_prompt_array(
#if NeedFunctionPrototypes
	void
#endif
);

void   
set_prmompt_array_cursor(
#if NeedFunctionPrototypes
	Cursor cursor
#endif
);

void
update_prompt_array(
#if NeedFunctionPrototypes
	void
#endif
);

int
promptn(
#if NeedFunctionPrototypes
	int num,
	int id,
	char *sta,
	char *stb,
	char *st0,
	char *st1,
	char *st2,
	char *st3,
	char *st4,
	char *st5
#endif
);

void
warning(
#if NeedFunctionPrototypes
	char *str1,
	char *str2
#endif
);

int
prompt_cont(
#if NeedFunctionPrototypes
	char *str1,
	char *str2
#endif
);

char
*prompt_str(
#if NeedFunctionPrototypes
	char *sta,
	char *stb
#endif
);

void
unmap_prompt(
#if NeedFunctionPrototypes
	void
#endif
);

void
display_msg(
#if NeedFunctionPrototypes
	char *str
#endif
);

void
expose_display_msg(
#if NeedFunctionPrototypes
	Window iwin
#endif
);

void
map_prompt(
#if NeedFunctionPrototypes
	void
#endif
);

void
win_str_prompt(
#if NeedFunctionPrototypes
	int opt,
	char *str
#endif
);

void
clear_win_str_prompt(
#if NeedFunctionPrototypes
	int opt
#endif
);

void
prompt_func(
#if NeedFunctionPrototypes
	int num,
	int typ,
	char *str1, 
	char *str2,
	char * (*str_label)(),
	void (*end_fn)(),
	char *st0,
	void (*fn0)(),
	char *st1,
	void (*fn1)(),
	char *st2,
	void (*fn2)(),
	char *st3,
	void (*fn3)(),
	char *st4,
	void (*fn4)()
#endif
);

/*************************************************
	GLOBAL PROTOTYPES: menusel.c
*************************************************/

void
init_select_item(
#if NeedFunctionPrototypes
	Display		*idisp,
	Window		iwin,
	GC		imgc, 
	GC		irv_mgc,
	GC		ilight_gc,
	XFontStruct	*ifont_info,
	long		iforeground,
	long		ibackground,
	int		ilogo_width,
	int		ilogo_height,
	int		width,
	menu_def	*def,
	int		inum_buttons,
	int		menusize,
	int		idebug
#endif
);

void
handle_select_button_event(
#if NeedFunctionPrototypes
	XEvent *report,
	int *last_button
#endif
);

void
update_panel_array(
#if NeedFunctionPrototypes
	int width
#endif
);


void
win_str(
#if NeedFunctionPrototypes
	int opt,  
	char *str
#endif
);

void
clear_win_str(
#if NeedFunctionPrototypes
	int opt  
#endif
);

void
clear_win_str_status(
#if NeedFunctionPrototypes
	int opt,
	menu_item *status
#endif
);

void
win_str_status(
#if NeedFunctionPrototypes
	int opt,
	char *str,
	menu_item *status
#endif
);


void
draw_select_item(
#if NeedFunctionPrototypes
	menu_item *item
#endif
);

void
draw_unselect_item(
#if NeedFunctionPrototypes
	menu_item *item
#endif
);

menu_item
*load_menu_def(
#if NeedFunctionPrototypes
	menu_def *def,
	int num
#endif
);

void
update_button(
#if NeedFunctionPrototypes
	int mainmenu_item,
	int submenu_item,
	int value
#endif
);

int
det_menu_index(
#if NeedFunctionPrototypes
	int menu,
	int submenu
#endif
);

int
det_menu_key_index(
#if NeedFunctionPrototypes
	int keyindex
#endif
);

void
process_menu_func(
#if NeedFunctionPrototypes
	int keyindex,
	FILE *tracefile
#endif
);

void
enable_menu_item(
#if NeedFunctionPrototypes
	void
#endif
);
 
void
disable_menu_item(
#if NeedFunctionPrototypes
	void
#endif
);

int
is_menu_item_disable(
#if NeedFunctionPrototypes
	void
#endif
);

void
set_select_button(
#if NeedFunctionPrototypes
	int menu,
	int button,
	int enable
#endif
);

/*************************************************
	GLOBAL PROTOTYPES: cursor.c
*************************************************/

void
set_cursor(
#if NeedFunctionPrototypes
	int cursor
#endif
);

void
reset_cursor(
#if NeedFunctionPrototypes
	void
#endif
);
 

/*************************************************
	GLOBAL PROTOTYPES: key.c
*************************************************/

void
init_key_debug_select(
#if NeedFunctionPrototypes
	int value
#endif
);

void
init_keylist(
#if NeedFunctionPrototypes
	void
#endif
);
 
void
add_key(
#if NeedFunctionPrototypes
	KeySym keysym,
	int key_type,
	int menu_id,
	void (*subroutine)(),
	int control,
	int meta,
	int alt,
	int report_param,
	int debug
#endif
);

void
add_menu_key(
#if NeedFunctionPrototypes
	Key_menu_def *menu_keys,
	int size
#endif
);
 
void
add_debug_key(
#if NeedFunctionPrototypes
	KeySym keysym,           
	void (*subroutine)()
#endif
);
 
void
handle_key_press(
#if NeedFunctionPrototypes
	XEvent *report
#endif
);
 
void
disable_keys(
#if NeedFunctionPrototypes
	void
#endif
);
 
void
enable_keys(
#if NeedFunctionPrototypes
	void
#endif
);


void
handle_key_release(
#if NeedFunctionPrototypes
	XEvent *report
#endif
);

/*************************************************
        GLOBAL PROTOTYPES: coord.c
*************************************************/

void
set_snap(
#if NeedFunctionPrototypes
	 Rtyp x,
	 Rtyp y
#endif
);

void
snapxy(
#if NeedFunctionPrototypes
	 Rtyp *rxp,
	 Rtyp *ryp,
	 int snap_type
#endif
);

void
ptor_xfrm(
#if NeedFunctionPrototypes
	int pixelx,
	int pixely,
	Rtyp *userxp,
	Rtyp *useryp,
	int use_snap,
	int snap_type
#endif
);

void 
rtop_xfrm(
#if NeedFunctionPrototypes
	Rtyp userxp,
	Rtyp useryp,
	int *pixelx,
	int *pixely
#endif
);

#ifdef DOS
/*************************************************
        GLOBAL PROTOTYPES: parsecmd.c

	NOTE: Needed because desqview/X Xrmparsecommand has a bug in it.

*************************************************/

void 
ssXrmParseCommand(
#if NeedFunctionPrototypes
    XrmDatabase *pdb,
    register XrmOptionDescList options,
    int num_options,
    char *prefix,
    int *argc,
    char **argv
#endif
);

#endif /* DOS */

#endif  /* End: __sx11_h */
